@@ -25,12 +25,15 @@
 (define-module (nyacc lang c99 cpp)
   #:export (
 	    cpp-line->stmt
+	    find-incl-in-dirl
 	    eval-cpp-cond-text
 	    expand-cpp-macro-ref
+	    ;;
+	    expand-cpp-name
 	    parse-cpp-expr
-	    find-incl-in-dirl
+	    eval-cpp-expr
 	    scan-arg-literal
-	    eval-cpp-expr)
+	    )
   #:use-module (nyacc parse)
   #:use-module (nyacc lex)
   #:use-module (nyacc lang sx-util)
@@ -42,6 +45,8 @@
   (else
    (use-modules (ice-9 optargs))
    (use-modules (nyacc compat18))))
+(define (sferr fmt . args)
+  (apply simple-format (current-error-port) fmt args))
 
 (define c99-std-defs
   '("__DATE__" "__FILE__" "__LINE__" "__STDC__" "__STDC_HOSTED__"
@@ -159,9 +164,9 @@
 		  ((read-c-ident ch))
 		  ((cpp-comm-skipper ch) (loop (skip-il-ws (read-char))))
 		  (else (throw 'cpp-error "bad #define")))))
-	 (args (or (p-args (read-char)) '()))
+	 (args (p-args (read-char)))
 	 (repl (p-rest (skip-il-ws (read-char)))))
-    (if (pair? args)
+    (if args
 	`(define (name ,name) (args . ,args) (repl ,repl))
 	`(define (name ,name) (repl ,repl)))))
 	
@@ -284,6 +289,7 @@
 	    ((pre-dec post-dec) (1- (ev1 tree)))
 	    ((pos) (ev1 tree))
 	    ((neg) (- (ev1 tree)))
+
 	    ((not) (if (zero? (ev1 tree)) 1 0))
 	    ((mul) (* (ev1 tree) (ev2 tree)))
 	    ((div) (/ (ev1 tree) (ev2 tree)))
@@ -483,11 +489,14 @@
      ((char=? #\( ch)
       (let loop2 ((argl argl) (argv '()) (ch ch))
 	(cond
-	 ((eqv? ch #\)) (reverse argv))
-	 ((null? argl) (cpp-err "arg count"))
-	 ((and (null? (cdr argl)) (string=? (car argl) "..."))
+	 ((eof-object? ch) (throw 'cpp-error "EOF reading args"))
+	 ((char=? ch #\))
+	  (reverse (if (and (pair? argl) (string=? (car argl) "..."))
+		       (acons "__VA_ARGS__" "" argv)
+		       argv)))
+	 ((and (pair? argl) (string=? (car argl) "..."))
 	  (let ((val (scan-cpp-input defs used #\))))
-	    (loop2 (cdr argl) (acons "__VA_ARGS__" val argv) (read-char))))
+	    (loop2 '() (acons "__VA_ARGS__" val argv) (read-char))))
 	 ((or (char=? ch #\() (char=? ch #\,))
 	  (let* ((val (scan-cpp-input defs used #\,)))
 	    (loop2 (cdr argl) (acons (car argl) val argv) (read-char))))
@@ -591,7 +600,7 @@
 ;; @end example
 ;; @noindent
 ;; Note that this routine will look in the current-input so if you want to
-;; expand text, 
+;; expand text, BE CAREFUL AND USE (with-input-from-string "" ...)
 ;; @end deffn
 (define* (expand-cpp-macro-ref ident defs #:optional (used '()))
   (let ((rval (assoc-ref defs ident)))
@@ -618,4 +627,12 @@
      ((c99-std-val ident) => identity)
      (else #f))))
 
+;; @deffn {Procedure} expand-cpp-name name defs => repl|#f
+;; Calls @code{expand-cpp-macro-ref} with null input string (w/o further
+;; input).  If @var{name} is has a function definition @code{#f} is returned.
+;; @end deffn
+(define (expand-cpp-name name defs)
+  (with-input-from-string ""
+    (lambda () (expand-cpp-macro-ref name defs))))
+
 ;;; --- last line ---
